==> 0.11.3.tar.gz: test/common_routes_test.dart <==
library ac_dart.addon_routebuilder.common.test;

import 'package:test/test.dart';
import 'package:mockito/mockito.dart';

import 'package:atlassian_connect_shelf/router.dart';
import 'package:atlassian_connect_shelf/acdart_app.dart';
import 'package:mojito/mojito.dart' show ClientId, printRoutes;
import 'package:shelf/shelf.dart';
import 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart';
import 'dart:convert';
import 'dart:async';
import 'package:uuid/uuid.dart';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:atlassian_connect_shelf/src/config/config.dart';

class MockRouter extends Mock implements Router {
  noSuchMethod(Invocation invocation) => super.noSuchMethod(invocation);
}

class MockAcHostService extends Mock implements AcHostService {
  noSuchMethod(Invocation invocation) => super.noSuchMethod(invocation);
}

Logger _log = new Logger('test');

main() {
  Logger.root.level = Level.OFF;
  const String secret = 'shhhhhh';
  String productHostKey;
  const String addonKey = 'foo-bar-addon';

  // TODO: it sucks testing the routes with the whole acdart initialised.
  // Need to refactor to avoid.
  final app = init(
      'https://localhost:9999', new AcDartHostRepoConfig.inMemory(),
      isDevMode: () => true);

  app.router.addons.bitbucket(addonKey, (_) => new ClientId('myid', 'mysecret'),
      addonSecret: 'shhhhhhhhhhhhhhhhhh')
    ..addInstallationRoute()
    ..addSpaRedirect('spa', 'moiAwsomePage');

  printRoutes(app.router, printer: _log.info);

  final routerHandler = app.router.handler;

  setUp(() {
    productHostKey = new Uuid().v1();
  });

  Future<Response> handleInstall() async {
    final installEvent = new InstallationEvent(new AcHostRegistration(
        10, new AcHostDetails.build(key: productHostKey), secret));
    return await routerHandler(new Request(
        'POST', Uri.parse('http://foo.bar/bitbucket/installed'),
        body: JSON.encode(installEvent)));
  }

  group('addInstallationRoute', () {
    Response response;
    setUp(() async {
      response = await handleInstall();
    });

    test('returns non null response', () async {
      expect(response, isNotNull);
    });
    test('returns 200 status code', () async {
      expect(response.statusCode, equals(200));
    });
  });

  group('addSpaRedirect', () {
    String jwt;
    Uri requestedUri;
    Response response;
    String location;
    Uri locationUri() => Uri.parse(location);
    String path() => locationUri().path;
    String fragmentQuery() {
      final fragment = Uri.decodeComponent(locationUri().fragment);
      return fragment.substring(fragment.indexOf('?') + 1);
    }
    Map fragmentParams() => Uri.splitQueryString(fragmentQuery());
    JsonWebToken sessionToken() => new JsonWebToken.decode(
        fragmentParams()['sessionToken'],
        claimSetParser: (j) => new AddonSessionClaimSet.fromJson(j));
    AddonSessionClaimSet claimSet() => sessionToken().claimSet;

    setUp(() async {
      await handleInstall();

      jwt = createProductHostToken('bitbucket/spa/app.html', 'GET', {
        'foo': 'blah'
      }, secret, productHostKey, subject: 'fred');

      requestedUri =
          Uri.parse('https://foo.bar/bitbucket/spa/app.html?jwt=$jwt&foo=blah');
      response = await routerHandler(new Request('GET', requestedUri));

      location = response.headers['location'];
    });

    test('returns 301', () {
      expect(response.statusCode, equals(301));
    });

    test('sets location header', () {
      expect(location, isNotNull);
    });

    test('location has same scheme', () {
      expect(locationUri().scheme, equals(requestedUri.scheme));
    });

    test('location has same authority', () {
      expect(locationUri().authority, equals(requestedUri.authority));
    });

    test('location has same port', () {
      expect(locationUri().port, equals(requestedUri.port));
    });

    test('path has changed base as expected', () {
      expect(path(), equals('/ui/spa/app.html'));
    });

    test('location has no query params', () {
      expect(locationUri().queryParameters, isEmpty);
    });

    test('location has a fragment', () {
      expect(locationUri().fragment, isNotEmpty);
    });

    test('location has expected number of fragment params', () {
      expect(fragmentParams(), hasLength(2));
    });

    test('location includes provided query params in fragment', () {
      expect(fragmentParams(), containsPair('foo', 'blah'));
    });

    test('location includes sessionToken in fragment', () {
      expect(fragmentParams().keys, contains('sessionToken'));
    });

    test('sessionToken issuer is addon', () {
      expect(claimSet().issuer, equals(addonKey));
    });

    test('sessionToken productHostKey is correct', () {
      expect(claimSet().productHostKey, equals(productHostKey));
    });

    test('sessionToken subject is correct', () {
      expect(claimSet().subject, equals('fred'));
    });
  });
}

==> 0.11.6.tar.gz: test/common_routes_test.dart <==
library ac_dart.addon_routebuilder.common.test;

import 'dart:async';
import 'dart:convert';

import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart';
import 'package:atlassian_connect_shelf/acdart_app.dart';
import 'package:atlassian_connect_shelf/router.dart';
import 'package:atlassian_connect_shelf/src/config/config.dart';
import 'package:logging/logging.dart';
import 'package:mockito/mockito.dart';
import 'package:mojito/mojito.dart' show ClientId, printRoutes;
import 'package:shelf/shelf.dart';
import 'package:test/test.dart';
import 'package:uuid/uuid.dart';

class MockRouter extends Mock implements Router {}

class MockAcHostService extends Mock implements AcHostService {}

Logger _log = new Logger('test');

main() {
  Logger.root.level = Level.OFF;
  const String secret = 'shhhhhh';
  String productHostKey;
  const String addonKey = 'foo-bar-addon';

  // TODO: it sucks testing the routes with the whole acdart initialised.
  // Need to refactor to avoid.
  final app = init(
      'https://localhost:9999', new AcDartHostRepoConfig.inMemory(),
      isDevMode: () => true);

  app.router.addons.bitbucket(addonKey, (_) => new ClientId('myid', 'mysecret'),
      addonSecret: 'shhhhhhhhhhhhhhhhhh')
    ..addInstallationRoute()
    ..addSpaRedirect('spa', 'moiAwsomePage');

  printRoutes(app.router, printer: _log.info);

  final routerHandler = app.router.handler;

  setUp(() {
    productHostKey = new Uuid().v1();
  });

  Future<Response> handleInstall() async {
    final installEvent = new InstallationEvent(new AcHostRegistration(
        10, new AcHostDetails.build(key: productHostKey), secret));
    return await routerHandler(new Request(
        'POST', Uri.parse('http://foo.bar/bitbucket/installed'),
        body: JSON.encode(installEvent)));
  }

  group('addInstallationRoute', () {
    Response response;
    setUp(() async {
      response = await handleInstall();
    });

    test('returns non null response', () async {
      expect(response, isNotNull);
    });
    test('returns 200 status code', () async {
      expect(response.statusCode, equals(200));
    });
  });

  group('addSpaRedirect', () {
    String jwt;
    Uri requestedUri;
    Response response;
    String location;
    Uri locationUri() => Uri.parse(location);
    String path() => locationUri().path;
    String fragmentQuery() {
      final fragment = Uri.decodeComponent(locationUri().fragment);
      return fragment.substring(fragment.indexOf('?') + 1);
    }
    Map fragmentParams() => Uri.splitQueryString(fragmentQuery());
    JsonWebToken sessionToken() =>
        new JsonWebToken.decode(fragmentParams()['sessionToken'],
            claimSetParser: (j) => new AddonSessionClaimSet.fromJson(j));
    AddonSessionClaimSet claimSet() => sessionToken().claimSet;

    setUp(() async {
      await handleInstall();

      jwt = createProductHostToken('bitbucket/spa/app.html', 'GET',
          {'foo': 'blah'}, secret, productHostKey,
          subject: 'fred');

      requestedUri =
          Uri.parse('https://foo.bar/bitbucket/spa/app.html?jwt=$jwt&foo=blah');
      response = await routerHandler(new Request('GET', requestedUri));

      location = response.headers['location'];
    });

    test('returns 301', () {
      expect(response.statusCode, equals(301));
    });

    test('sets location header', () {
      expect(location, isNotNull);
    });

    test('location has same scheme', () {
      expect(locationUri().scheme, equals(requestedUri.scheme));
    });

    test('location has same authority', () {
      expect(locationUri().authority, equals(requestedUri.authority));
    });

    test('location has same port', () {
      expect(locationUri().port, equals(requestedUri.port));
    });

    test('path has changed base as expected', () {
      expect(path(), equals('/ui/spa/app.html'));
    });

    test('location has no query params', () {
      expect(locationUri().queryParameters, isEmpty);
    });

    test('location has a fragment', () {
      expect(locationUri().fragment, isNotEmpty);
    });

    test('location has expected number of fragment params', () {
      expect(fragmentParams(), hasLength(2));
    });

    test('location includes provided query params in fragment', () {
      expect(fragmentParams(), containsPair('foo', 'blah'));
    });

    test('location includes sessionToken in fragment', () {
      expect(fragmentParams().keys, contains('sessionToken'));
    });

    test('sessionToken issuer is addon', () {
      expect(claimSet().issuer, equals(addonKey));
    });

    test('sessionToken productHostKey is correct', () {
      expect(claimSet().productHostKey, equals(productHostKey));
    });

    test('sessionToken subject is correct', () {
      expect(claimSet().subject, equals('fred'));
    });
  });
}
